@@ -36,6 +36,7 @@ class ScriptEditorWidget;
 class KAboutData;
 class QAction;
 class KToggleAction;
+class KSelectAction;
 class QProgressDialog;
 
 namespace Cantor{
@@ -83,6 +84,8 @@ Q_SIGNALS:
     void setCaption(const QString& caption, const QIcon& icon);
     void showHelp(const QString& help);
     void worksheetSave(const QUrl& url);
+    void requestOpenWorksheet(const QUrl& url);
+    void setBackendName(const QString& name);
 
 public Q_SLOTS:
     void updateCaption();
@@ -116,6 +119,8 @@ protected Q_SLOTS:
     void exportToLatex();
     void evaluateOrInterrupt();
     void restartBackend();
+    void zoomValueEdited(const QString& text);
+    void updateZoomWidgetValue(double zoom);
     void enableTypesetting(bool enable);
     void showBackendHelp();
     void print();
@@ -169,6 +174,8 @@ private:
     bool m_showProgressDlg;
     QAction * m_evaluate;
     QAction * m_restart;
+    KSelectAction* m_zoom;
+    QAction* m_currectZoomAction;
     QAction * m_save;
     QAction * m_findNext;
     QAction * m_findPrev;
